<?php

/**
 * @file
 * Contains \Drupal\username_reminder\Form\UsernameReminderForm.
 */

namespace Drupal\username_reminder\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\user\UserStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a username reminder form.
 */
class UsernameReminderForm extends FormBase {

  /**
   * The user storage.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a UserPasswordForm object.
   *
   * @param \Drupal\user\UserStorageInterface $user_storage
   *   The user storage.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(UserStorageInterface $user_storage, LanguageManagerInterface $language_manager) {
    $this->userStorage = $user_storage;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.manager')->getStorage('user'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'username_reminder_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['instructions'] = array(
      '#prefix' => '<p>',
      '#markup' => $this->t('Complete the form below and an e-mail will be sent to you with your username.'),
      '#suffix' => '</p>',
    );
    $form['email'] = array(
      '#type' => 'email',
      '#title' => $this->t('Email address'),
      '#required' => TRUE,
    );
    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array('#type' => 'submit', '#value' => $this->t('Email your username'));
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $email = trim($form_state->getValue('email'));
    $users = $this->userStorage->loadByProperties(array('mail' => $email, 'status' => 1));
    $account = reset($users);
    if ($account && $account->id()) {
      $form_state->setValueForElement(array('#parents' => array('account')), $account);
    }
    else {
      $form_state->setErrorByName('email', $this->t('Sorry, %email is not a recognized email address.', array('%email' => $email)));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $account = $form_state->getValue('account');
    $langcode = $this->languageManager->getCurrentLanguage()->getId();
    if (!$langcode) {
      $langcode = $account->getPreferredLangcode();
    }
    $params['account'] = $account;
    // Get the custom site notification email to use as the from email address
    // if it has been set.
    $site_mail = \Drupal::config('system.site')->get('mail_notification');
    // If the custom site notification email has not been set, we use the site
    // default for this.
    if (empty($site_mail)) {
      $site_mail = \Drupal::config('system.site')->get('mail');
    }
    if (empty($site_mail)) {
      $site_mail = ini_get('sendmail_from');
    }

    // Mail one time login URL and instructions using current language.
    $mail = \Drupal::service('plugin.manager.mail')->mail('username_reminder', 'reminder', $account->getEmail(), $langcode, $params, $site_mail);
    if (!empty($mail)) {
      $this->logger('user')->notice('Username reminder mailed to %name at %email.', array('%name' => $account->getUsername(), '%email' => $account->getEmail()));
      drupal_set_message($this->t('Your username has been sent to your email address.'));
    }

    $form_state->setRedirect('user.page');
  }

}
